#!/usr/bin/env python

"""
Convert and optimise images for display in an Acorn Electron MODE 1 variant
with four colours per line but eight colours available for selection on each
line.

Copyright (C) 2015 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from optimiserlib import *
from os.path import split, splitext
import EXIF
import PIL.Image
import sys

# Image operations.

def test():

    "Generate slices of the colour cube."

    size = 512
    for r in (0, 63, 127, 191, 255):
        im = PIL.Image.new("RGB", (size, size))
        for g in range(0, size):
            for b in range(0, size):
                value = get_value((r, (g * 256) / size, (b * 256 / size)))
                im.putpixel((g, b), value)
        im.save("rgb%d.png" % r)

def test_flat(rgb):

    "Generate a flat image for the colour 'rgb'."

    size = 64
    im = PIL.Image.new("RGB", (size, size))
    for y in range(0, size):
        for x in range(0, size):
            im.putpixel((x, y), get_value(rgb))
    im.save("rgb%02d%02d%02d.png" % rgb)

def rotate_and_scale(exif, im, width, height, rotate):

    """
    Using the given 'exif' information, rotate and scale image 'im' given the
    indicated 'width' and 'height' constraints and any explicit 'rotate'
    indication. The returned image will be within the given 'width' and
    'height', filling either or both, and preserve its original aspect ratio.
    """

    if rotate or exif and exif["Image Orientation"].values == [6L]:
        im = im.rotate(270)

    w, h = im.size
    if w > h:
        height = (width * h) / w
    else:
        width = (height * w) / h

    return im.resize((width, height))

def get_float(options, flag):
    try:
        i = options.index(flag)
        try:
            return float(options[i+1])
        except (IndexError, ValueError):
            return 1.0
    except ValueError:
        return 0.0

# Main program.

if __name__ == "__main__":

    # Test options.

    if "--test" in sys.argv:
        test()
        sys.exit(0)
    elif "--test-flat" in sys.argv:
        test_flat((120, 40, 60))
        sys.exit(0)
    elif "--help" in sys.argv:
        print >>sys.stderr, """\
Usage: %s <input filename> <output filename> [ <options> ]

Options are...

-s - Saturate the input image (can be followed by a float, default 1.0)
-d - Desaturate the input image (can be followed by a float, default 1.0)
-D - Darken the input image (can be followed by a float, default 1.0)
-B - Brighten the input image (can be followed by a float, default 1.0)

-r - Rotate the input image clockwise
-p - Generate a separate preview image
-h - Make the preview image with half horizontal resolution (MODE 2)
-v - Verify the output image (loaded if -n is given)
-n - Generate no output image
""" % split(sys.argv[0])[1]
        sys.exit(1)

    width = 320
    height = 256

    input_filename, output_filename = sys.argv[1:3]
    basename, ext = splitext(output_filename)
    preview_filename = "".join([basename + "_preview", ext])

    options = sys.argv[3:]

    # Preprocessing options that can be repeated for extra effect.

    saturate = get_float(options, "-s")
    desaturate = get_float(options, "-d")
    darken = get_float(options, "-D")
    brighten = get_float(options, "-B")

    # General output options.

    rotate = "-r" in options
    preview = "-p" in options
    half_resolution_preview = "-h" in options
    verify = "-v" in options
    no_normal_output = "-n" in options
    make_image = not no_normal_output

    # Load the input image if requested.

    if make_image or preview:
        exif = EXIF.process_file(open(input_filename))
        im = PIL.Image.open(input_filename).convert("RGB")
        im = rotate_and_scale(exif, im, width, height, rotate)

        sim = SimpleImage(list(im.getdata()), im.size)
        process_image(sim, saturate, desaturate, darken, brighten)
        im.putdata(sim.getdata())

    # Generate a preview if requested.

    if preview:
        imp = im.copy()
        sim = SimpleImage(list(im.getdata()), im.size)
        simp = preview_image(sim, half_resolution_preview)
        imp.putdata(simp.getdata())
        imp.save(preview_filename)

    # Generate an output image if requested.

    if make_image:
        sim = SimpleImage(list(im.getdata()), im.size)
        convert_image(sim)
        im.putdata(sim.getdata())
        im.save(output_filename)

    # Verify the output image (which may be loaded) if requested.

    if verify:
        if no_normal_output:
            im = PIL.Image.open(output_filename).convert("RGB")

        im = SimpleImage(list(im.getdata()), im.size)
        result = count_colours(im, 4)
        if result is not None:
            y, colours = result
            print "Image %s: row %d has the following colours: %s" % (output_filename, y, "; ".join([repr(c) for c in colours]))

# vim: tabstop=4 expandtab shiftwidth=4
